/**************************************************************************
**  (c) Copyright 1999, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : date.h
**      SYSTEM NAME    : AXE - Andromeda X-windows Encapsulation
**      VERSION NUMBER : $Revision: 1.3 $
**
**  DESCRIPTION      :  
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 06, 1998
**      LAST UPDATE     : Oct 16, 1999
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: date.h,v $
   Revision 1.3  2002-09-28 06:58:45  arjen
   Bugfix: conversion of an empty string to a date or hour object
   now makes the values of such an object 0 (null) instead of giving
   a segmentation fault.
   The class UTC combines the date and hour classes. The most basic
   functions of the UTC class are now implemented.
   These include constructors and conversion to and from String objects.
   New functions: date::proper(), hour::proper() and UTC::proper().
   Return true if the object holds a proper clock time and/or calendar
   date; false if at least one value is out of range.

   Revision 1.2  2002/09/02 06:18:20  arjen
   Fixed some date and time conversion functions

   Revision 1.1  2002/07/25 08:01:26  arjen
   First checkin, AXE release 0.2

*****************************/

/* static const char *RCSID = "$Id: date.h,v 1.3 2002-09-28 06:58:45 arjen Exp $"; */

#ifndef AXE_DATE_H
#define AXE_DATE_H

#include "String.h"

class date
{
   unsigned char  month, day;
   short year;

   long julian();
   date add(date D);
   date add(unsigned long days);

   date subtract(unsigned long days);

   friend class UTC;

public:

   date()
   {
      month = 0;
      day   = 0;
      year  = 0;
   }       
   
   date(unsigned  d, unsigned  m=0, short y=0)
   {
      day   = d;
      month = m;
      year  = y;
   }

   date(String s);

   bool proper();   // Check wether this is a proper calendar date

   friend date operator+(date, date);
   friend date operator+(unsigned long,  date);
   friend date operator+(date,  unsigned long);

   date operator += (date D)
   {
      return add(D);
   }

   date operator += (unsigned long l)
   {
      return add(l);
   }

   date operator ++ ()        // prefix
   {
      return add(1);
   }

   date operator ++ (int)     // postfix
   {
      return add(1);
   }

   friend date operator-(unsigned long,  date);
   friend date operator-(date,  unsigned long);
   friend long operator-(date&, date&);

   date operator -= (unsigned long l)
   {
      return subtract(l);
   }

   date operator -- ()        // prefix
   {
      return subtract(1);
   }

   date operator -- (int)     // postfix
   {
      return subtract(1);
   }

//   long(date);        // Calculate nr. of days since...

   int operator==(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm == 0;
   }

   int operator!=(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm != 0;
   }             
   
   int operator<=(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm <= 0;
   }             
   
   int operator>=(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm >= 0;
   }             
   
   int operator>(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm > 0;
   }             
   
   int operator<(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm < 0;
   }

   //  Attributes.

   unsigned Day()
   {
      return day;
   }

   unsigned Month()
   {
      return month;
   }

   String MonthName(void);

   int Year()
   {
      return year;
   }
   //  Operations.

   int Leap(void);
   unsigned DaysInMonth();
   unsigned YearDay();
   int WeekDay()
   {
      return julian() % 7 + 1;
   }
    // TODO:YearDay WeekDay WeekNumber DayName

//   date operator*(double)
//   date& operator*=(double)
//   date operator/(double)
//   date& operator/=(double)

   friend std::ostream& operator<<(std::ostream&, const date&);
   friend std::istream& operator>>(std::istream&, date&);

   String format(const char *fmt = "%F");
};

date today(bool localtime = true); 

class hour
{
   int   hours;
   short minutes, seconds;

   friend class UTC;

   int compare(hour h)
   {
      int cm;
      
      cm = this->hours - h.hours;
      cm = (cm != 0) ? cm : this->minutes - h.minutes;
      cm = (cm != 0) ? cm : this->seconds - h.seconds;
      
      return cm;
   }

   hour increase();
   hour decrease();

public:

   hour()
   {
      hours = 0;
      minutes = 0;
      seconds = 0;
   }

   hour(int hr, short minute, short sec)
   {
      hours = hr;
      minutes = minute;
      seconds = sec;
   }

   hour(long sec)
   {
      hours = sec / 3600;
      minutes = sec / 60 % 60;
      seconds = sec % 60;
   }

   hour(String s);

   bool proper();   // Check wether this is a proper clock time

   //  Attributes.

   short Second()
   {
      return seconds;
   }

   short Minute()
   {
      return minutes;
   }

   int Hour()
   {
      return hours;
   }

   bool operator==(hour h)
   {
      return compare(h) == 0;
   }

   bool operator!=(hour h)
   {
      return compare(h) != 0;
   }

   bool operator<(hour h)
   {
      return compare(h) < 0;
   }

   bool operator>(hour h)
   {
      return compare(h) > 0;
   }

   bool operator<=(hour h)
   {
      return compare(h) <= 0;
   }

   bool operator>=(hour h)
   {
      return compare(h) >= 0;
   }

   friend hour operator+(hour, hour);
   friend hour operator+(hour, long);
   hour operator += (hour h);

   hour operator ++ ()        // prefix
   {
      return increase();
   }

   hour operator ++ (int)     // postfix
   {
      return increase();
   }

   friend hour operator-(hour, hour);
   friend hour operator-(hour, long);
   hour operator -= (hour h);

   hour operator -- ()        // prefix
   {
      return decrease();
   }

   hour operator -- (int)     // postfix
   {
      return decrease();
   }


   friend std::ostream& operator<<(std::ostream &, const hour &);
   friend std::istream& operator>>(std::istream &, hour &);

   String format(const char *fmt = "%T");
};

hour now(bool localtime = true);

class UTC
{
   hour  t;
   date  d;

public:

   UTC()
   {
      /* Leave everything to the default hour and date constructors */
   }

   UTC(const String s);
   UTC(date dt, hour tm)
   {
      d = dt;
      t = tm;
   }

   operator date()
   {
      return d;
   }

   operator hour()
   {
      return t;
   }

   bool proper()
   {
      return d.proper() && t.proper();
   }

   bool operator==(UTC u)
   {
      return d == u.d && t == u.t;
   }

   bool operator!=(UTC u)
   {
      return d != u.d || t != u.t;
   }

   bool operator<=(UTC u)
   {
      if (d == u.d)
      {
         return t <= u.t;
      }
      else
      {
         return d <= u.d;
      }
   }

   bool operator<(UTC u)
   {
      if (d == u.d)
      {
         return t < u.t;
      }
      else
      {
         return d < u.d;
      }
   }

   bool operator>=(UTC u)
   {
      if (d == u.d)
      {
         return t >= u.t;
      }
      else
      {
         return d >= u.d;
      }
   }

   bool operator>(UTC u)
   {
      if (d == u.d)
      {
         return t > u.t;
      }
      else
      {
         return d > u.d;
      }
   }

   friend UTC operator+(UTC &, UTC &);
   UTC operator += (UTC h);

   friend std::ostream& operator<<(std::ostream &, const UTC &);
   friend std::istream& operator>>(std::istream &, UTC &);

   String format(const char *fmt = "%F %T");
};

UTC Now(bool localtime = true);

#endif /* AXE_DATE_H */
